/**
 * @file boost/mirror/pre_registered/type/native.hpp
 * @brief Pre-registration of the native C++ types with Mirror
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_MIRROR_PRE_REGISTERED_TYPE_NATIVE_1011291729_HPP
#define BOOST_MIRROR_PRE_REGISTERED_TYPE_NATIVE_1011291729_HPP

#include <boost/mirror/meta_namespace.hpp>
#include <boost/mirror/meta_class.hpp>
// pointers and references
#include <boost/mirror/pre_registered/type/_ptr_ref.hpp>
// const and volatile types
#include <boost/mirror/pre_registered/type/_const_volatile.hpp>
// array types
#include <boost/mirror/pre_registered/type/_array.hpp>
// function types
#include <boost/mirror/pre_registered/type/_free_fn.hpp>

BOOST_MIRROR_REG_BEGIN

BOOST_MIRROR_REG_GLOBAL_SCOPE_TYPE_CTS(void, ('v','o','i','d'))

// helper macro for registering the native C++ types
// besides the basic type information is registered
// the default and copy constructors are also registered
#define BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(TYPE, SPELLED) \
BOOST_MIRROR_REG_GLOBAL_SCOPE_TYPE_CTS(TYPE, SPELLED) \
namespace _class { \
template <> \
struct _< TYPE > : _defaults_native< TYPE, boost::mirror::spec_type_tag > \
{ }; \
}

BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        bool,
        ('b','o','o','l')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        char,
        ('c','h','a','r')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        unsigned char,
        ('u','n','s','i','g','n','e','d',' ','c','h','a','r')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        wchar_t,
        ('w','c','h','a','r','_','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        short int,
        ('s','h','o','r','t',' ','i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        int,
        ('i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        long int,
        ('l','o','n','g',' ','i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        unsigned short int,
        ('u','n','s','i','g','n','e','d',' ','s','h','o','r','t',' ','i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        unsigned int,
        ('u','n','s','i','g','n','e','d',' ','i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        unsigned long int,
        ('u','n','s','i','g','n','e','d',' ','l','o','n','g',' ','i','n','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        float,
        ('f','l','o','a','t')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        double,
        ('d','o','u','b','l','e')
)
BOOST_MIRROR_REG_NATIVE_TYPE_HELPER(
        long double,
        ('l','o','n','g',' ','d','o','u','b','l','e')
)

#undef BOOST_MIRROR_REG_NATIVE_TYPE_HELPER

BOOST_MIRROR_REG_END

#endif //include guard

